<?php declare(strict_types=1);
/*
 * This file is part of PHPUnit.
 *
 * (c) Sebastian Bergmann <sebastian@phpunit.de>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
use SebastianBergmann\Environment\Runtime;

require_once __DIR__ . '/../../vendor/autoload.php';

if (!(new Runtime)->hasPCOV()) {
    print 'skip: This test requires PCOV to be available for code coverage';
}
